#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: PFB Channelizer Perf
# Author: Arnaud Durand
##################################################

from gnuradio import blocks
from gnuradio import analog
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.filter import pfb
import filter_opencl
import argparse

GSM_RATE = 1625000.0 / 6.0
nchannels = 50

class top_block(gr.top_block):

    def __init__(self):
        gr.top_block.__init__(self, "Top Block")

        ##################################################
        # Variables
        ##################################################
        self.nchannels = nchannels
        self.transition_width = transition_width = 10e3
        self.samp_rate = samp_rate = GSM_RATE*nchannels
        self.gain = gain = 30.9
        self.cutoff_freq = cutoff_freq = 135e3
        self.pfb_taps = pfb_taps = firdes.low_pass(gain, samp_rate, cutoff_freq, transition_width, firdes.WIN_HANN)
        print len(pfb_taps)
        self.center_freq = center_freq = 0

        ##################################################
        # Blocks
        ##################################################
        self.pfb_channelizer_ccf_0 = filter_opencl.pfb_channelizer_ccf(
        	  nchannels,
        	  (pfb_taps))
        #self.pfb_channelizer_ccf_0.set_channel_map(())
        	
        self.blocks_null_sink_0 = blocks.null_sink(gr.sizeof_gr_complex*1)
        self.analog_sig_source_0 = analog.sig_source_c(samp_rate, analog.GR_COS_WAVE, 1000, 1)
        self.blocks_head_0 = blocks.head(gr.sizeof_gr_complex*1, int(samp_rate*60))

        self.s2ss = blocks.stream_to_streams(gr.sizeof_gr_complex*1, self.nchannels)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_sig_source_0, 0), (self.blocks_head_0, 0))
        self.connect((self.blocks_head_0, 0), (self.s2ss, 0))
        for i in range(self.nchannels):
            self.connect((self.s2ss, i), (self.pfb_channelizer_ccf_0, i))
            self.connect((self.pfb_channelizer_ccf_0, i), (self.blocks_null_sink_0, i))


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("nchannels", help="benchmark PFB channelizer for nchannels",
                        type=int, default=50)
    args = parser.parse_args()
    nchannels = args.nchannels
    tb = top_block()
    import time
    start = time.time()
    tb.start()
    tb.wait()
    end = time.time()
    print "%.4gs" % (end-start)
