#ifndef INCLUDED_volk_32fc_32f_dot_prod_32fc_avx2_H
#define INCLUDED_volk_32fc_32f_dot_prod_32fc_avx2_H

#include <volk/volk_common.h>
#include<stdio.h>
#include <immintrin.h>


static inline void volk_32f_x2_dot_prod_32f_a_avx2(float* result, const  float* input, const  float* taps, unsigned int num_points) {
  unsigned int number = 0;
  const unsigned int eighthPoints = num_points / 8;

  float dotProduct = 0;
  const float* aPtr = input;
  const float* bPtr = taps;

  __m256 aVal;
  __m256 bVal;

  __m256 cVal = _mm256_setzero_ps();

  for(;number < eighthPoints; number++){

    aVal = _mm256_load_ps(aPtr);
    bVal = _mm256_load_ps(bPtr);

    cVal = _mm256_fmadd_ps(aVal, bVal, cVal);

    aPtr += 8;
    bPtr += 8;
  }

  __VOLK_ATTR_ALIGNED(32) float dotProductVector[8];

  _mm256_store_ps(dotProductVector, cVal); // Store the results back into the dot product vector

  dotProduct = dotProductVector[0];
  dotProduct += dotProductVector[1];
  dotProduct += dotProductVector[2];
  dotProduct += dotProductVector[3];
  dotProduct += dotProductVector[4];
  dotProduct += dotProductVector[5];
  dotProduct += dotProductVector[6];
  dotProduct += dotProductVector[7];

  number = eighthPoints*8;
  for(;number < num_points; number++){
    dotProduct += ((*aPtr++) * (*bPtr++));
  }

  *result = dotProduct;
}

static inline void volk_32fc_32f_dot_prod_32fc_a_avx2(lv_32fc_t* result, const lv_32fc_t* input, const float* taps, unsigned int num_points) {
  unsigned int number = 0;
  const unsigned int eighthPoints = num_points / 8;

  float res[2];
  float *realpt = &res[0], *imagpt = &res[1];
  const float* aPtr = (float*)input;
  const float* bPtr = taps;

  __m256 a0Val, a1Val;
  __m256 b0Val, b1Val;
  __m256 xVal, xloVal, xhiVal;

  __m256 cVal = _mm256_setzero_ps();

  for(;number < eighthPoints; number++){

    a0Val = _mm256_load_ps(aPtr);
    a1Val = _mm256_load_ps(aPtr+8);

    xVal = _mm256_load_ps(bPtr); // t0|t1|t2|t3|t4|t5|t6|t7
    xloVal = _mm256_unpacklo_ps(xVal, xVal); // t0|t0|t1|t1|t4|t4|t5|t5
    xhiVal = _mm256_unpackhi_ps(xVal, xVal); // t2|t2|t3|t3|t6|t6|t7|t7

    b0Val = _mm256_permute2f128_ps(xloVal, xhiVal, 0x20); // t0|t0|t1|t1|t2|t2|t3|t3
    b1Val = _mm256_permute2f128_ps(xloVal, xhiVal, 0x31); // t4|t4|t5|t5|t6|t6|t7|t7

    cVal = _mm256_fmadd_ps(a0Val, b0Val, cVal);
    cVal = _mm256_fmadd_ps(a1Val, b1Val, cVal);

    aPtr += 16;
    bPtr += 8;
  }

  __VOLK_ATTR_ALIGNED(32) float dotProductVector[8];

  _mm256_store_ps(dotProductVector, cVal); // Store the results back into the dot product vector

  *realpt = dotProductVector[0];
  *imagpt = dotProductVector[1];
  *realpt += dotProductVector[2];
  *imagpt += dotProductVector[3];
  *realpt += dotProductVector[4];
  *imagpt += dotProductVector[5];
  *realpt += dotProductVector[6];
  *imagpt += dotProductVector[7];

  number = eighthPoints*8;
  for(;number < num_points; number++){
    *realpt += ((*aPtr++) * (*bPtr));
    *imagpt += ((*aPtr++) * (*bPtr++));
  }

  *result = *(lv_32fc_t*)(&res[0]);
}

#endif

