#include <sys/types.h>
#include <vector>

#include <gnuradio/top_block.h>
#include <gnuradio/filter/firdes.h>
#include <gnuradio/filter/pfb_channelizer_ccf.h>
#include <gnuradio/blocks/api.h>
#include <gnuradio/blocks/null_sink.h>
#include <gnuradio/blocks/null_source.h>
#include <gnuradio/blocks/file_source.h>
#include <gnuradio/blocks/file_sink.h>
#include <gnuradio/blocks/vector_source_c.h>
#include <gnuradio/blocks/vector_sink_c.h>
#include <gnuradio/blocks/stream_to_streams.h>

#include <unistd.h>

#include <filter_avx2/pfb_channelizer_ccf.h>
#define GSM_RATE (1625000.0 / 6.0)
#include <complex.h>

using namespace std;

int main(int argc, char **argv)
{
    int nchannels=5;
    // set filter
    float v[12] = {0,1,2,3,4,5,6,7,8,9,10,11};
    vector<float> lowpass (v,v+12);

    gr_complex vv[24] = {{23, 23000},{22, 22000},{21, 21000},{20, 20000},{19, 19000},{18, 18000},{17, 17000},{16, 16000},{15, 15000},{14, 14000},{13, 13000},{12, 12000},{11, 11000},{10, 10000},{9, 9000},{8, 8000},{7, 7000},{6, 6000},{5, 5000},{4, 4000},{3, 3000},{2, 2000},{1, 1000},{0, 0}};
    vector<gr_complex> input (vv,vv+24);
    gr::filter::firdes *grfilter;

    printf("[%f:%f]\n",vv[11].real(),vv[11].imag());

    double transition_width = 10e3;
    double samp_rate = GSM_RATE*nchannels;
    double gain = 30.9;
    double cutoff_freq = 135e3;
    //lowpass = grfilter->low_pass(gain, samp_rate, cutoff_freq, transition_width);
    gr::top_block_sptr tb = gr::make_top_block("bench");
    printf("lowpass: [%lu]\n",lowpass.size());

    //gr::blocks::null_source::sptr source = gr::blocks::null_source::make(sizeof(gr_complex));
    //gr::blocks::file_source::sptr source = gr::blocks::file_source::make(sizeof(gr_complex), "/media/duranda/1A1016A610168941/gsm_to_Arnaud_11ch_959MHz_30s.dump", false);
    gr::blocks::vector_source_c::sptr source = gr::blocks::vector_source_c::make(input);


    gr::blocks::null_sink::sptr sink = gr::blocks::null_sink::make(sizeof(gr_complex));
    //vector<gr_complex> sinks;
    vector<gr::blocks::vector_sink_c::sptr> sinks;
    for(int i=0; i<nchannels; i++)
        sinks.push_back(gr::blocks::vector_sink_c::make());


   gr::blocks::stream_to_streams::sptr s2ss = gr::blocks::stream_to_streams::make(sizeof(gr_complex), nchannels);
    gr::filter_avx2::pfb_channelizer_ccf::sptr pfb = gr::filter_avx2::pfb_channelizer_ccf::make(nchannels, lowpass, 1.0f);

    tb->connect(source, 0, s2ss, 0);
    for(int i=0; i<nchannels; i++){
      tb->connect(s2ss, i, pfb, i);
      tb->connect(pfb, i, sinks[i], 0);
    }


    tb->start();
    tb->wait();

    for(int i=0; i<sinks.size(); i++){
    printf("\nSINKS[%i]: ",i);
    for(int j=0; j<sinks[i]->data().size(); j++)
      printf("%f|",sinks[i]->data()[j].real());
      }
}

