/* -*- c++ -*- */
/*
 * Copyright 2015 <Arnaud Durand>.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_FILTER_OPENCL_PFB_CHANNELIZER_CCF_IMPL_H
#define INCLUDED_FILTER_OPENCL_PFB_CHANNELIZER_CCF_IMPL_H

#include <filter_opencl/pfb_channelizer_ccf.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef __APPLE__
#include <OpenCL/opencl.h>
#else
#include <CL/cl.h>
#endif
#include <clFFT.h>

#define VECTOR_SIZE 8192

//OpenCL kernel which is run for every workitem created.
const char *fir_kernel =
"__kernel void fir_kernel(__global float2 *A, __constant float *B, __global float2 *C, int delay_len) \n"
"{ \n"
"    //Get the index of the work-item \n"
"    int filter = get_global_id(0); \n"
"    int delay = get_global_id(1); \n"
"    \n"
"    float2 dotProduct = (float2)( 0, 0 ); \n"
"    \n"
"    __global float2 *delay0 = A + (8192+delay_len) * filter + delay; \n"
"    __constant float *w = B + delay_len * filter; \n"
"    \n"
"    for(int i=0; i<delay_len; i++){ \n"
"      dotProduct = /*fma*/mad(delay0[i], w[i], dotProduct); \n"
"    } \n"
"    \n"
"    int nfilts = get_global_size (0); \n"
"    C[nfilts*delay + filter] = dotProduct; \n"
"} \n"
;

namespace gr {
  namespace filter_opencl {

    class pfb_channelizer_ccf_impl : public pfb_channelizer_ccf
    {
     private:
      float* d_taps;
      unsigned int d_taps_per_filter;
      unsigned int d_nfilts;
      std::vector<int> d_channel_map;
      gr::thread::mutex     d_mutex; // mutex to protect set/work access

      cl_int err;

      cl_platform_id * platforms = NULL;
      cl_uint num_platforms;
      cl_device_id *device_list = NULL;
      cl_uint num_devices;
      cl_program program;
      cl_kernel kernel;

      cl_context context;
      cl_command_queue command_queue;

      cl_mem in_clmem;
      cl_mem taps_clmem;
      cl_mem out_clmem;

      clfftPlanHandle plan_handle;

      cl_mem inplace_buffers[VECTOR_SIZE];
      //cl_buffer_region inplace_regions[VECTOR_SIZE];

      size_t global_size[2]; // Process the entire lists
      size_t local_size;

      //float *in_ptr;
      gr_complex *out_ptr;

      void set_taps(const std::vector<float> &taps);

     public:
      pfb_channelizer_ccf_impl(unsigned int nfilts, const std::vector<float> &taps);
      ~pfb_channelizer_ccf_impl();

      // Where all the action really happens
      //void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      void set_channel_map(const std::vector<int> &map);
      std::vector<int> channel_map() const;

      int general_work(int noutput_items,
		       gr_vector_int &ninput_items,
		       gr_vector_const_void_star &input_items,
		       gr_vector_void_star &output_items);
    };

  } // namespace filter_opencl
} // namespace gr

#endif /* INCLUDED_FILTER_OPENCL_PFB_CHANNELIZER_CCF_IMPL_H */

